/**
 * MySDL - A multimedia framework based on SDL and OpenGL
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://www.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <assert.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "files.h"

#ifdef win32
	namespace mysdl
	{
		// ================================================================
		//  Common functions
		// ================================================================

		/*
		* See header file for documentation.
		*/
		bool mkdirs(std::string path)
		{
			return(false);
		}

		// ================================================================
		//  SysDirManager
		// ================================================================

		/*
		* See header file for documentation.
		*/
		SysDirManager::SysDirManager(std::string applicationName,SysDirCategory category) :
				applicationName(applicationName),category(category),constDataDir()
		{
		}

		/*
		* See header file for documentation.
		*/
		std::string SysDirManager::getSubDir(SysDir type,std::string subDir,bool create)
		{
			if(type==sdConstData)
			{
				if(constDataDir.length()>0)
				{
					return constDataDir;
				}
			}
			
			return("");//./"+subDir+"/");
		}

		/*
		* See header file for documentation.
		*/
		std::string SysDirManager::getDir(SysDir dirType)
		{
			if(dirType==sdConstData)
			{
				if(constDataDir.length()>0)
				{
					return constDataDir;
				}
			}
			
			return("");//./");
		}
	};

#else
	#include <pwd.h>

	namespace mysdl
	{
		// ================================================================
		//  Common functions
		// ================================================================

		/*
		* See header file for documentation.
		*/
		bool mkdirs(std::string path)
		{
			if(path.size()==0)
			{
				return(true);
			}

			if(path[path.size()-1]!='/')
			{
				for(int i=path.size()-1; i>=0; i--)
				{
					if(path[i]=='/')
					{
						path=path.substr(0,i);
						break;
					}
				}

				if(path.size()==0)
				{
					return(true);
				}
			}

			struct stat statBuffer;

			for(unsigned int i=1; i<path.size(); i++)
			{
				if(path[i]=='/')
				{
					std::string dir=path.substr(0,i);

					if(stat(dir.c_str(),&statBuffer)==0)
					{
							//mysdl::debug::debug(dbgResources,"Directory exists: %s",dir.c_str());
						continue;
					}

					if(errno==ENOENT)
					{
							//mysdl::debug::debug(dbgResources,"Creating directory: %s",dir.c_str());
						if(mkdir(dir.c_str(),0770)==-1) // TODO: correct mask?
						{
								//mysdl::debug::debug(dbgResources,"Directory creation failed.");
							return(false);
						}
					}
					else
					{
						return(false);
					}
				}
			}

			return(true);
		}

		// ================================================================
		//  SysDirManager
		// ================================================================

		/*
		* See header file for documentation.
		*/
		SysDirManager::SysDirManager(std::string applicationName,SysDirCategory category) :
				applicationName(applicationName),category(category),constDataDir()
		{
		}

		/*
		* See header file for documentation.
		*/
		std::string SysDirManager::getSubDir(SysDir type,std::string subDir,bool create)
		{
			std::string dir=getDir(type)+subDir;

			if((dir.size()>0)&&(dir[dir.size()-1]!='/'))
			{
				dir+'/';
			}

			if((create)&&(dir.length()>0))
			{
				mkdirs(dir);
			}

			//mysdl::debug::debug(dbgResources,"dir=%s",dir.c_str());

			return(dir);
		}

		/*
		* See header file for documentation.
		*/
		std::string SysDirManager::getDir(SysDir dirType)
		{
			switch(dirType)
			{
				case sdConfig:
				{
					return(std::string("/etc/")+applicationName+"/");
				}

				case sdUser:
				{
					struct passwd *userData=getpwuid(getuid());

					std::string dir=std::string(userData->pw_dir);

					if(dir[dir.size()-1]!='/')
					{
						dir+='/';
					}

					return(dir+"."+applicationName+"/");
				}

				case sdConstData:
				{
					if(constDataDir.length()>0)
					{
						return constDataDir;
					}
					
					switch(category)
					{
						case sdGame:
							return(std::string("/usr/share/games/")+applicationName+std::string("/"));

						default:
							return(std::string("/usr/share/")+applicationName+std::string("/"));
					}
				}

				case sdVarData:
				{
					switch(category)
					{
						case sdGame:
							return(std::string("/var/games/")+applicationName+std::string("/"));

						default:
							return(std::string("/var/lib/")+applicationName+std::string("/"));
					}
				}
			}

			assert(false);
		}
	};
#endif
